# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
from enum import Enum


class ProcessControllerSourceSensor(Enum):
    """
    Servo Tuning Parameter Set to target.
    """

    THERMISTOR = 10
    ANALOG_INPUT = 20
