# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
from enum import Enum


class OscilloscopeDataSource(Enum):
    """
    Kind of channel to record in the Oscilloscope.
    """

    SETTING = 0
    IO = 1
